/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1995, 1996, 1997, 1998 John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@johncon.com (John Conover)

------------------------------------------------------------------------------

translit.c, transliterate a record

void transliterate (unsigned char *record);

    translate the characters in record using uppercase and soundex as
    translation tables

The soundex algorithm is a mechanical phonetic translation system for
the English language, and converts English words into a corresponding
phonetic code for the word. The algorithm is as follows:

    for each character in a word:

        if the character is the first character of a word

            1) do nothing

        else

            2) replace consecutive sequences of the labials, (ie., the
            characters, B, F, P, V,) with the character '1'

            3) replace consecutive sequences of the gutterals and
            sibilants, (ie., the characters, C, G, J, K, Q, S, X, Z,)
            with the character '2'

            4) replace consecutive sequences of the dentals, (ie., the
            characters, D, T,) with the character '3'

            5) replace consecutive sequences of the longliquids, (ie.,
            the character, L,) with the character '4'

            6) replace consecutive sequences of the nasals, (ie., the
            characters, M, N,) with the character '5'

            7) replace consecutive sequences of the shortliquids,
            (ie., the character, R,) with the character '6'

            8) and, omit all other characters, (ie., the characters,
            A, E, H, I, O, U, W, Y,)

            9) if the soundex translation of the word is larger than 4
            characters, truncate to 4 characters.

    For example, the soundex translation of the word "conover" is
    C516. Extended word length searches can be enhanced by eliminating
    9), above.

    Running the above algorithm on a standard text version of the
    Webster's English dictionary, (mine has 234,932 words,) results in
    61,408 different words being recognized by the algorithm.

The algorithm is as follows:

    for each character in the record

        replace the character with its equivilent in uppercase[]
        and/or soundex[]

Usage is a call with record referencing the first character to be
translated

    transliterate (record);

There are no errors, and nothing is returned

To test this module, compile the module source with -DTEST_TRANSLIT

$Revision: 1.0 $
$Date: 1998/03/09 00:29:06 $
$Id: translit.c,v 1.0 1998/03/09 00:29:06 john Exp $
$Log: translit.c,v $
Revision 1.0  1998/03/09 00:29:06  john
Initial revision


*/

#include "pdict.h"

#ifndef LINT /* include rcsid only if not running lint */

static char rcsid[] = "$Id: translit.c,v 1.0 1998/03/09 00:29:06 john Exp $"; /* module version */
static char rcsid_h[] = TRANSLIT_H_ID; /* module include version */

#endif

#ifdef __STDC__

void transliterate (unsigned char *record)

#else

void transliterate (record)
    unsigned char *record;

#endif

{
    unsigned char *record_ref = record, /* reference to current character being translated in record */
                  *soundex_ref = record;  /* reference to current soundex character in record */

    *record_ref = uppercase[(int) *record_ref]; /* the first character is the upper case of the first character */
    record_ref ++; /* next current character being translated in record */
    soundex_ref ++; /* next current soundex character being translated in record */

    while (*record_ref != (unsigned char) '\0') /* for each additional character in record */
    {
        *soundex_ref = soundex[(int) *record_ref]; /* translate the character to soundex */

        if ((*soundex_ref != (unsigned char) '\0') && (*soundex_ref != *(soundex_ref - 1))) /* if the soundex translation was not a vowel, H, W, or Y, or the previous soundex character was not the same as the current soundex character */
        {
            soundex_ref ++; /* accept the soundex character */
        }

        record_ref ++; /* next current character being translated in record */
    }

    *soundex_ref = (unsigned char) '\0'; /* terminate the soundex word with an EOS */
}

#ifdef TEST_TRANSLIT

/*

simple exerciser for testing transliterate (); get a string from
stdin, transliterate it, and print it to stdout; ignore the:

declared global, could be static
    transliterate       translit.c(xx)

from lint

*/

#ifdef __STDC__

int main (void)

#else

int main ()

#endif

{
    unsigned char record[BUFSIZ]; /* word to be translated to soundex */

    make_uppercase (); /* setup the uppercase array */

    while (gets (record) != (char *) 0) /* while there are words/records on standard input */
    {
        transliterate (record); /* transliterate the record */
        (void) printf ("%s\n", record); /* print the soundex transliterated record */
    }

    exit (0); /* return success */

#ifdef LINT /* include only if running lint */

    return (0); /* for LINT formality */

#endif

}

#endif
